package HSPC::Request;
use strict;

use SWsoft::protect;

package HSPC::Request::Query; ## Apache::Request pretending CGI for backward compatibility
use base qw(Apache::Request);

use Apache::Constants qw(REDIRECT);

## ~ Apache::Request->instance
sub instance($$) {
	my $class = shift;
	my $r = shift;

	## bless Apache::Request object to HSPC::Request::Query package
	my $self = bless $class->SUPER::instance($r), $class;

	return $self;
}

## ~ CGI::cgi_error
sub cgi_error($) {
	my $self = shift;

	return $self->parse ? $self->notes('error-notes') : undef;
}

## ~ CGI::redirect
sub redirect($@) {
	my $self = shift;
	my %h = @_;

	$self->err_headers_out->add('Set-Cookie' => $_)
		foreach values %{$ENV{cookies_out}};
	$self->headers_out->set('Location' => $h{-uri});

	return REDIRECT;
}

## ~ CGI::url
sub url($) {
	my $self = shift;

	return $self->uri;
}

## ~ CGI::uploadInfo
sub uploadInfo($$) {
	my $self = shift;
	my $filename = shift;

	my ($upload) = grep {$_->filename eq $filename} $self->upload;

	return $upload ? $upload->info : undef;
}

## Get/set application error flag.
## => self
## => error
## <= error
sub error($;$) {
	my $self = shift;

	return $self->notes(request_error => @_);
}

## Add/get warning message (red line atop of every page).
## => self
## => warnings
## <= warnings
sub warnings($;$) {
	my $self = shift;

	my $warnings = $self->pnotes('request_warnings');
	if (@_) {
		foreach my $warning (@_) {
			push @$warnings, $warning
				unless grep { $_ eq $warning } @$warnings; ## skip duplicates
		}
		$self->pnotes(request_warnings => $warnings);
	}

	return $warnings || [];
}

## Send file contents and ignore teaser results.
## => self
## => {
##        filename	=> <scalar>
##        mime_type	=> <scalar>
##        content	=> <scalarref>
##    }
sub send_file($$) {
	my $self = shift;

	return $self->pnotes(request_send_file => @_);
}

package HSPC::Request;

use Apache::Constants qw(
	OK DECLINED REDIRECT FORBIDDEN HTTP_FORBIDDEN NOT_FOUND HTTP_NOT_FOUND SERVER_ERROR
);
use Apache::Cookie ();
use Apache::URI ();

use HSPC::CompRep ();
use HSPC::Config ();
use HSPC::Localization ();

use HSPC::Util::String qw(sw_decode_base64);
use HSPC::Logger qw(sw_log_warn sw_die);
use HSPC::Profiler ();
use HSPC::WebErrors ();
use HSPC::WebSystem ();
use HSPC::WebSystemLib qw(sw_uriparam sw_param sw_web_server_protocol
	sw_web_redirect sw_web_redirect_local sw_web_parse_template);
use HSPC::MT::GM ();
use HSPC::MT::GM::ProviderConfig;
use HSPC::MT::UP::ServiceMonSetup();

use constant PKG_DIR	=> '/usr/lib/perl5/site_perl/5.8.8';

sub handler {
	my $r = HSPC::Request::Query->instance(shift,
		POST_MAX => $HSPC::Config::CONFIG{MAX_FILE_UPLOAD_SIZE});
	my $is_rewriter = $r->current_callback eq 'PerlTransHandler';
	my $is_location = $r->location eq &HSPC::CompRep::CR_LINK_LOCATION;

	## backward compatibility: pretend CGI::Cookie
	$ENV{cookies_in} = Apache::Cookie->fetch;

	## set custom error documents depending on current language
	{
		my $error_document_dir =
			'/frontend/'
			. ($ENV{cookies_in}->{lng} && $ENV{cookies_in}->{lng}->value
				|| $HSPC::Localization::LOCALE{local_lang}
				|| $HSPC::Localization::LOCALE{def_lang})
			. '/';
		$r->custom_response(FORBIDDEN,
			$error_document_dir . Apache::Constants::HTTP_FORBIDDEN . '.shtml');
		$r->custom_response(NOT_FOUND,
			$error_document_dir . Apache::Constants::HTTP_NOT_FOUND . '.shtml');
		$r->custom_response(SERVER_ERROR,
			$error_document_dir . Apache::Constants::SERVER_ERROR . '.shtml');

	}

	## skip PerlTransHandler for PATH_INFO processing of CGI requests
	return DECLINED
		if $is_rewriter && !$r->is_initial_req;

	## remove '/hspc', if any
	$ENV{REQUEST_URI} = $ENV{PATH_INFO} = $is_location
		? substr($r->uri, 5)
		: $r->uri;
	## extract tool ID and request URI
	my (undef, $tool, $uri, $uri_params) = split('/', $ENV{REQUEST_URI}, 4);
	undef $uri
		if $ENV{PATH_INFO} =~ s/^.+\.cgi//i; ## prepare for old-fashioned CGI request

	## initialize environment
	$ENV{BASEURL} = join('/', $is_location ? $r->location : '', $tool);
	$ENV{CC_BASEURL} = $ENV{BASEURL}
		if &HSPC::CompRep::CR_TOOL_TO_DIR_MAP->{$tool} eq 'cc';

	## could be some file request
	unless (&HSPC::CompRep::CR_TOOL_TO_DIR_MAP->{$tool}) {
		$r->uri($ENV{REQUEST_URI}) ## remove location from URI and proceed
			if $is_location;
		return DECLINED;
	}

	## get repository instance, set $ENV{COMPREP}
	$ENV{COMPREP} = eval {HSPC::CompRep->new(
		tool	=> $tool,
		cache	=> !$HSPC::Config::CONFIG{COMPREP_NOCACHE},
	)};
	if ($@ && !$is_rewriter) {
		warn sprintf('[%s] HSPC::CompRep->new error: %s',
			__PACKAGE__, $@);
		return SERVER_ERROR;
	}

	## do URI translation during corresponding phase
	if ($is_rewriter) {
		## remove location from URI and proceed if old-fashioned CGI request found
		$r->uri($ENV{REQUEST_URI})
			if index($ENV{REQUEST_URI}, '.cgi') > $[;
		return DECLINED;
	}

	## backward compatibility: pretend CGI and CGI::Cookie
	$ENV{q} = $r;
	$ENV{cookies_out} = {};
	$ENV{args} = $ENV{q}->param;

	## lookup node
	my $node;
	if ($uri =~ /^\d\d(?:\.\d{2,3})*$/) {
		## by URI
		$node = $ENV{COMPREP}->node(uri => $uri, choose => 1)
			or return NOT_FOUND;
	} elsif ($is_location && $uri =~ /^[\w-]+$/) {
		## by alias (to redirect)
		$node = $ENV{COMPREP}->node(alias => $uri)
			and return $r->redirect(-uri => join('/', $r->location, $tool, $node->{uri}, $uri_params));
	}

	## N.B.: local $node could be modified below, but it is not set as current!
	if ($node) {
	 	## try to locate node basing on 'tab' URI param, if any
		$node = $ENV{COMPREP}->node(alias => $node->{alias} . '_tab_' . $1)
				|| $node
			while $ENV{PATH_INFO} =~ m!/tab,([^/]+)!g;
	 	## try to locate node basing on 'act' URI param, if any
		$node = $ENV{COMPREP}->node(alias => $node->{alias} . '_act_' . $1)
				|| $node
			while $ENV{PATH_INFO} =~ m!/a(?:ct)?,([^/]+)!g;

		## lookup tab/action, if any, and fake URI params
		my $parent = $ENV{COMPREP}->node_parent($node);
		## try to determine action basing on node alias
		if ($parent && $node->{alias} =~ s/^(\Q$parent->{alias}\E.*)_act_(.+)$/$1/) {
			my $act = $2;
			$node->{alias} .= "_act_$act"
				unless $ENV{COMPREP}->node(alias => $node->{alias});
			$ENV{PATH_INFO} .= '/' . join('/',
				map {"$_,$act"}			## fake parameter
				grep {!sw_uriparam($_)}	## check whether parameter exists already
				qw/act a/);				## list of parameters to fake
		}
		## try to determine tab
		$ENV{PATH_INFO} .= "/tab,$1"
			if $parent
				&& index($node->{alias}, $parent->{alias}) == $[
				&& ($parent->{alias} =~ /_tab_(.+)$/
					|| $node->{alias} =~ /^\Q$parent->{alias}\E_tab_(.+?)(_act_.+)?$/);
	} else {
		undef $uri;
	}

	## initialize system object
	delete $ENV{$_}
		foreach qw(security_obj system_obj session);
	eval {HSPC::WebSystem->run(1,
		LOGIN_REQUIRED	=> index($node->{alias}, 'login') < $[
						&& index($node->{alias}, 'logout') < $[
						&& $tool ne 'common',
		init_person		=> $tool ne 'common',
		access_for_hold	=> 1,
	)};
	my $error = $@;
	return REDIRECT
		if $r->headers_out->{Location};
	sw_die(sprintf('[%s] HSPC::WebSystem->run error: %s',
			__PACKAGE__, $error))
		if $error;

	## switch SSL or on off if needed
	if ($ENV{system_obj}) {
		my $is_ssl = HSPC::MT::GM->get_ssl_state($ENV{system_obj}->provider_id,
			$ENV{COMPREP}->{tool}, $node);
		return $r->redirect(-uri => 'http' . ($is_ssl ? 's' : '') . '://'
				. $r->hostname . $r->uri . ($r->args ? '?' . $r->args : ''))
			if $is_ssl && sw_web_server_protocol() ne 'https' && length($r->uri) > 1
				|| !$is_ssl && sw_web_server_protocol() eq 'https';
	}

	my $result;
	if (defined $uri) {
		my $error = undef;
		## initialize profiling
		my $prof = HSPC::Profiler::profiler(
			url		=> $r->parsed_uri->unparse,
			comment	=> $node->{alias}
		);
		## test security for node
		if ($tool ne 'common') {
			my $is_access = eval {$ENV{security_obj}->test_security($node->{alias})};
			$error = $@;
			return FORBIDDEN
				unless $error || $is_access;

			## manage browsing history	
			if (!(ref $ENV{ session }->{aliases_history})
			|| index($uri_params, 'menu_path') >= $[ 
			){
				@{$ENV{ session }->{aliases_history}} = ();
			}
			my $index;
			for ($index = 0; $index < @{$ENV{ session }->{aliases_history}}; $index++) {
 			   last if ${$ENV{ session }->{aliases_history}}[$index]->{alias} eq $node->{alias};
	        }    
	        splice (@{$ENV{ session }->{aliases_history}}, $index) if defined $index;
	        push (@{$ENV{ session }->{aliases_history}}, 
	        	{ 
	        		alias		=> $node->{alias}, 
	        	  	uri_params	=> $uri_params
	        	}
	        );
		}
		## call URI handler to process the result
		unless ($error) {
			$result = eval {
				local *CORE::GLOBAL::exit = sub {
					## fight against Apache::exit, for it kills HSPC::Exc on occasion
					my (undef, $filename, $line) = caller;
					die "exit() called at $filename line $line";
				};
				(my $package = "$node->{class}.pm") =~ s!::!/!g;
				require $package;
				return ($node->{class})->${\$node->{method}};
			};
			$error = $@;
		}
		return REDIRECT
			if $r->headers_out->{Location};
		if (!$error && ref $result eq 'HASH' && $result->{STATUS}) {
			## wrap results with common page template
			my $warnings = join('<br>', @{$r->warnings});
			my $failed_task_info = HSPC::MT::TM::Queue->failed_task_info();


			$result->{CONTENT} = sw_web_parse_template(__PACKAGE__, 'page_default.tmpl', {
				content			=> $result->{CONTENT},
				warnings		=> $ENV{COMPREP}->{tool} eq 'pcc'
					&& $warnings,
				service_alert	=> $ENV{COMPREP}->{tool} eq 'pcc'
					&& HSPC::UP::ServiceMon->GetAlertStatus && 1,
				up2date_status	=> $ENV{COMPREP}->{tool} eq 'pcc'
					&& HSPC::MT::UP->check_update,
				up2date_version	=> $ENV{COMPREP}->{tool} eq 'pcc'
					&& HSPC::MT::UP::Setup->new->hspc_last_version,
				task_failed		=> $ENV{COMPREP}->{tool} eq 'pcc'
					&& $failed_task_info->{count},
				task_failed_first_time => $ENV{COMPREP}->{tool} eq 'pcc'
					&& $failed_task_info->{first_time},
				warn_disk_usage		=> $ENV{COMPREP}->{tool} eq 'pcc'
					&& HSPC::MT::UP::ServiceMonSetup->check_disk_full,
			}) if $result->{STATUS} eq 'OK' && !$result->{NO_WRAP};

			## customization per alias
			my $path_node = $node;
			while ($path_node) {
				if (-f &PKG_DIR . '/HSPC/Custom/Screen/' . $path_node->{alias} . '.pm') {
					my $class = 'HSPC::Custom::Screen::' . $path_node->{alias};
					eval {
						no strict 'refs';
						require 'HSPC/Custom/Screen/' . $path_node->{alias} . '.pm';
						$result->{CONTENT} = &{$class . '::customize'}($result->{CONTENT}, $node->{alias});
					};
					sw_log_warn("customization error in $class: $@")
						if $@;
				}
				$path_node = $ENV{COMPREP}->node_parent($path_node);
			}
		} else {
			unless ($r->error) {
				## don't actually die here, but force problem report creation
				eval {
					sw_die(sprintf('URI handler (%s->%s) error: %s',
						$node->{class}, $node->{method},
						$error || ($result ? 'wrong' : 'no') . ' result returned'));
				};
				warn $@;
			}
			## proceed to error handler
			$result = {
				STATUS => 'ERROR',
				ERROR  => 'ERROR',
			};
			undef $result->{ERROR}
				if $r->error;
		}
	} else {
		## generate content for index page
		my $params = {w => sw_uriparam('w')};
		unless ($params->{w}) {
			$ENV{HSPC_SYSTEM}->upref_var($ENV{COMPREP}->{tool} . '_slider_closed',
					sw_uriparam('cs'))
				if defined sw_uriparam('cs');
			$params->{link_frame} = sw_param('frame_url')
				? sw_decode_base64(sw_param('frame_url'))
				: $ENV{COMPREP}->link(alias => 'cc_core_front');
			$params->{current_screen} = 'frameset.tmpl';
		} elsif ($params->{w} eq 'logo') {
			$params->{local_languages} = HSPC::Localization::get_languages();
			$params->{local_lang} = $HSPC::Localization::LOCALE{local_lang};
			$params->{current_screen} = 'logo.tmpl';
		} elsif ($params->{w} eq 'menu') {
			$params->{current_screen} = 'menu.tmpl';
		}
		$result = {
			STATUS	=> 'OK',
			CONTENT	=> sw_web_parse_template($HSPC::Config::CONFIG{OFFICEPATH} . '/template',
				$params->{current_screen}, $params),
			NO_HEAD	=> $params->{w} ne 'menu' && $params->{w} ne 'logo',
			NO_FOOT => 1,
		};
	}

	if (my $error = ($result->{STATUS} eq 'ERROR' && $result->{ERROR} || $r->error)) {
		## let frontend process error codes with static pages
		return FORBIDDEN
			if $error == 403;
		return NOT_FOUND
			if $error == 404;
		return SERVER_ERROR
			if $error == 500;

		## process error template on errors with dynamic pages
		$result = HSPC::WebErrors->teaser_error($error);
	}

	## strip template tags if needed
	my $show_tag = undef;
	if (ref($ENV{system_obj}->{_provider_config})) {
		$show_tag = $ENV{system_obj}->{_provider_config}->is_show_tmpl_tags();
	} else {
		$show_tag = HSPC::MT::GM::ProviderConfig->get_is_show_tmpl_tags(id => $ENV{system_obj}->provider_id());
	}
	$result->{CONTENT} =~ s#<!-- /?TAG type="[^"]+" id="[^"]+" -->##g
		if !$show_tag;

	## send HTTP responses
	if (ref $ENV{layout} eq 'HASH') {
		$ENV{system_obj}->{redirect} ||= $ENV{layout}->{redirect};
		delete $ENV{layout};
	}
	if ($result->{STATUS} eq 'REDIRECT') {
		## avoid calls to exit
		eval {
			$ENV{system_obj}->{redirect_local}
				? sw_web_redirect_local(url => $ENV{system_obj}->{redirect_local})
				: sw_web_redirect($ENV{system_obj}->{redirect});
		};
		return REDIRECT;
	}
	unless ($result->{STATUS} eq 'CUSTOM' || $result->{STATUS} eq 'POSTED') {
		$r->headers_out->set('Expires' => '-1d');
		$r->headers_out->add('Set-Cookie' => $_)
			foreach values %{$ENV{cookies_out}};
		## ignore content, if file was sent
		if (my $send_file = $r->send_file) {
			$result->{STATUS} = 'POSTED';
			$r->headers_out->set('Content-Disposition' => 'attachment; filename='
				. $send_file->{filename});
			$r->headers_out->set('Content-Length' => length ${$send_file->{content}});
			$r->send_http_header($send_file->{mime_type});
			print ${$send_file->{content}};
		} else {
			$r->send_http_header('text/html; charset=UTF-8');
		}
	}
	print HSPC::WebSystemLib::sw_web_html_header(
			no_body		=> $result->{NO_BODY},
			page_title	=> $ENV{HSPC_PAGE_TITLE},
		) if $result->{STATUS} eq 'OK' && !$result->{NO_HEAD};
	print $result->{CONTENT}
		if $result->{STATUS} ne 'POSTED' && $result->{STATUS} ne 'REDIRECT';

	return OK;
}

1;
