package HSPC::WebSystem;
use strict;

use HSPC::System;
use vars qw{ @ISA };

@ISA = qw(HSPC::System);

use HSPC::Config;
use HSPC::Localization; 
use HSPC::Session;
use HSPC::WebDB;
use CGI qw(:cgi-lib :standard);
use CGI::Cookie;
use CGI::Util qw(escape);
use Data::Dumper;
use HSPC::ErrorStack;
use HSPC::MT::Core;
use HSPC::MT::Core::Constants;
use HSPC::MT::CU;
use HSPC::MT::CU::Skin;
use HSPC::MT::GM;

use HSPC::Util qw(sw_find_custom_file);
use HSPC::Util::String qw(sw_encode_base64_safe);
use HSPC::Logger;
use HSPC::WebSecurity;
use HSPC::WebSystemLib;
use HTTP::BrowserDetect;
use IPC::ShareLite;
use SWsoft::protect;	# Mark this module for encryption upon build
use Text::Template;
use HSPC::LocResource;
use HSPC::MT::PM;
use HSPC::BM::Currency;
use POSIX;
use HSPC::DataCheck;

##### system constants #############

use constant SYSTEM_PRODUCT_ID => '01';
use constant SYSTEM_PRODUCT_CAPTION => 'Parallels Business Automation - Standard';
use constant SYSTEM_PRODUCT_VERSION => '3.0';
use constant TIPS_PREFIX=>'/var/opt/hspc-root/tool-tips/';
use constant EMPTY_IMAGE_URL=>'<img src="/images/1.gif">';

##### /system constants #############

my %cache_ui_templates; ## hash with Text::Template instances, keys of hash are template paths
my %cache_ui_templates_stat; ## hash with template attributes to check if it was changed
my %cache_ui_palettes; # hash with HSPC::MT::CU::SKin instances, hash keys are skin ID
my %cache_ui_palettes_as; # hash with HSPC::MT::CU::Skin IDs, hash keys are "vendorID|toolID"

#### shared value for UI data caching #####
my $ui_cache_share = new IPC::ShareLite(
	-key     => 234343,	# Random shared memory key
	-create  => 1,	# Create new SHM segment unless exixst
	-destroy => 0
) or die "Unable allocate shared memory $!";
$ui_cache_share->store(0) unless $ui_cache_share->fetch();
my $cache_ui_skin_check = $ui_cache_share->fetch();
#### /shared value for UI data caching #####


# Arguments
# > class
# >	full_init	1 = new app, 0 = old app
# >	SESSION 	(for old app)
# => init_person : init security object
# => tool_id : Current Tool ID, look hspc-sitelib/docs/ids.txt for details
# => tool_urlkey : set current tool_id from url by key = tool_urlkey
# => no_session : disable session initing
# => max_post_size : maximum allowed data size in POST requests.       
#      Needed in file namager and other modules where big data used.   
# => request_too_large_url : Local URL where user will be redirected if
#      user post data greater then value in "max_post_size" parameter  
# >= LOGIN_REQUIRED

### OUT-OF_DATE ARGUMENTS ####
# -> site_account_code   #REQ_N# out-of-date argument, drop it
# >= account_type        #REQ_Y# numeric | ref to call_back function returned account_type | types array
##############################
sub run {
	my $class = shift;
	my $full_init = shift;				

	##HSPC::Logger->debug(0,'[CORE:COMMON] web_init');

	my %args = (
		LOGIN_REQUIRED	=> 1,
		access_for_hold => 0,
		@_
	);        
	my $tool_id = defined $args{tool_urlkey} ? sw_uriparam($args{tool_urlkey}) : $args{tool_id};
    
	if($full_init) {

		# init CGI 
		$CGI::POST_MAX = $args{max_post_size} if $args{max_post_size};
		$ENV{q} ||= new CGI; ## backward compatibility for CGIs
		if ($ENV{q}->cgi_error =~ /^413\s/) {
		    close STDIN;
		    my $url = $args{request_too_large_url};
		    if ($url !~ /^http/) {
			$url = sw_web_server_protocol().'://'.&sw_web_server_name.$url;
		    }
		    return sw_web_redirect($url);
		}

		if ($ENV{q}->isa('CGI')) {
			$ENV{args} = $ENV{q}->Vars;
			$ENV{cookies_in} = CGI::Cookie->fetch unless $ENV{cookies_in};
			$ENV{cookies_out} = {};
		}

		# restore or create session
		if( (not defined $args{no_session}) && (not defined $ENV{session}) ){
			my $session = HSPC::Session::checkSession($ENV{cookies_in}, $ENV{cookies_out}, undef, $args{session_id});
			$ENV{session} = $session;
		}
	}
	else {
		$ENV{session} = shift;
		die 'CALL BAZA';
	}		
	
	my $system_obj = HSPC::WebSystem->new(tool_id=>$tool_id);
	
	## try to find provider
	$system_obj->provider_id();
	
	## init HTTP::BrowseDetect ##
	my $ua = $ENV{q}->can('user_agent') ? $ENV{q}->user_agent : $ENV{HTTP_USER_AGENT};
	$system_obj->{_browser} = HTTP::BrowserDetect->new($ua);
	$system_obj->{_browser}->{_hspc_useragent} = $ua;
		
##	!!!	HTTP/HTTPS selection routine moved to HSPC::Site::Layout::run
##		due to possible dependency on exact layout ('force_ssl' flag)

	# skip security checking for public sites #	
	if( (not $args{LOGIN_REQUIRED}) && (not $args{init_person})  ){				
		HSPC::Localization::set_language( $system_obj->locale_lang);
		$system_obj->_ui_init;
		return 1;
	}
	
	### store sec.params for future using
	$system_obj->{_access_for_hold} = $args{access_for_hold};
	$system_obj->{_login_required} = $args{LOGIN_REQUIRED};
	
	### init security	
	$system_obj->reinit_security(
		force_login=>$args{LOGIN_REQUIRED},		
	);
	my $security_obj = $ENV{security_obj};

	## Get skin object if not dedfined and store it in enviroment
	unless ($ENV{skin_obj}) {
		unless($security_obj->is_signed_in()) {
			my $provider_config = HSPC::MT::GM->find_provider_config(id => $system_obj->provider_id());
			if ($provider_config && $provider_config->def_skin()) {
				$ENV{skin_obj} = HSPC::MT::CU->find_skin( id => $provider_config->def_skin());
			}
		}
		else {
			$ENV{skin_obj} = HSPC::MT::CU->find_skin(person_id => $security_obj->person_id);
		}
	}
	
	my $pretender_key = $class->pretender_key($tool_id);
	# automated security check
	my $nok;
	if(ref($args{CATEGORY}) eq 'ARRAY'){
		foreach ( @{ $args{CATEGORY} } ) {
			$nok = !$security_obj->test_security($_); #TODO
			last unless $nok;
		}
	}elsif( defined $args{CATEGORY}){	
		$nok = !$security_obj->test_security($args{CATEGORY});
	}
	if ($nok) {
		HSPC::Logger->info(
			qq|LOGIN: can`t login into tool :| .
			$ENV{HSPC_SYSTEM}->tool_id .
			q|, person_id=| . $security_obj->person_id .
			q| pret=| . $ENV{session}->{$pretender_key}
		);
		sw_web_forbidden();
	}

	## Somebody comes in - Check was hspc configured
	if($security_obj->account_no) {
		my $acc_type = $security_obj->account_type;
		
		## provider did not comfigure HSPC
		if(!$system_obj->hsp_provider_config()->completed_full) {
			if($acc_type != SW_HSP) {
				## All other people except provider
				HSPC::Logger->warn('HSPC setup was not completed.');
				return &sw_web_forbidden();
			} elsif (
				$system_obj->tool_id eq 'pcc'
				&& (!$ENV{COMPREP}->node
					|| !grep { $ENV{COMPREP}->node->{alias} eq $_ } qw/
						initial_setup
						cc_core_richeditor
						trouble_tickets_request_support
						trouble_tickets_request_support_popup
					/)
			) {
				## redirect provider to configuration wizard
				return sw_web_redirect_comprep(alias => 'initial_setup');
			}
		}
		
		## if not provider
		if($security_obj->account_no != SW_HSP_ID) {
			my $provider_conf = $system_obj->provider_config();
			my $completed = $provider_conf->completed_full;
			my $provider = HSPC::MT::Core->find_account(account_id=>$provider_conf->id);
			if(!$completed) {
				if($acc_type == SW_RESELLER && $system_obj->tool_id eq 'rcc') {
					## do not redirect if reseller in CP
					if (!$ENV{COMPREP}->node
						|| !grep {$ENV{COMPREP}->node->{alias} eq $_} qw/
								initial_setup
								cc_core_richeditor
								trouble_tickets_request_support
								trouble_tickets_request_support_popup
							/) {
						## redirect reseller to configuration wizard
						return sw_web_redirect_comprep(alias => 'initial_setup');
					}
				}
				elsif($provider->type == SW_RESELLER) {
					HSPC::Logger->warn('HSPC setup for reseller was not completed.');
					return &sw_web_forbidden()
				}
			}
			
		}
	}
	
	### account selected, now we can init UI
	$system_obj->_ui_init;
	
	#### check account for status 'Active'
	if( 
		$system_obj->{_login_required}
		&& $ENV{security_obj}->account_no
		&& $ENV{security_obj}->account_status!=0 ## account is not active
		&& ! $system_obj->{_access_for_hold}
	){
		$system_obj->exit_acconhold();
	}
	
	return 1;
}

##############################################################################

# >> self
# << ref to HTTP::BrowserDetect instance
sub browser{
	my $self = shift;
	return $self->{_browser};
}

##############################################################################


sub _check_provider_server_name {
	my $self = shift;	
	my $q;
	if ( HSPC::DataCheck::is_ip(sw_web_server_name()) ) {
		$q = "SELECT id FROM gm_provider_config WHERE server_ip = ? ORDER BY id";
	} else {
		$q = "SELECT id FROM gm_provider_config WHERE server_name IS NOT NULL AND server_name= ?";
	}
	my $res = HSPC::WebDB::select_hash($q,&sw_web_server_name);			
	return undef unless defined $res;
	return $res->{id};
}

## >> self
## << provider_id or 0= can`t find provider by server_name
sub check_provider_server_name{
	my $self = shift;	
	my $id = _check_provider_server_name();
	return 0 unless defined $id;
	## store provider ID
	$ENV{HSPC_CU_SKIN_BID} = $id;	
	$ENV{VENDOR_SERVER_NAME}=&sw_web_server_name;
	return $id;
}

##############################################################################

# >> class
sub exit_noobject{		
	if ($ENV{q}->can('error')) {
		$ENV{q}->error('NOOBJECT');
		exit;
	}

	if( defined $ENV{layout} ){
		$ENV{layout}->set_error_noobject();
		exit;
	}

	use HSPC::Site::Layout;
	HSPC::Site::Layout->teaser_error('NOOBJECT');
		
	exit;
}

##############################################################################

# >> class
sub exit_acconhold{		
	if ($ENV{q}->can('error')) {
		$ENV{q}->error('ACCONHOLD');
		exit;
	}

	if( defined $ENV{layout} ){
		$ENV{layout}->set_error_acconhold();
		exit;
	}

	use HSPC::Site::Layout;
	HSPC::Site::Layout->teaser_error('ACCONHOLD');
		
	exit;
}


## This method just execute redirect, it does not make any security checks
## >> self
## => person
## => account
## => subscr_id
## => ve_id
## => tool_id
## => zone
## < {redirect result}
sub redirect_to_tool
{
	my $self = shift;
	my %h = @_;
	my $person_id = $h{person_id};
	my $account_no = $h{account_id} || $ENV{security_obj}->account_no();
	my $subscr_id = $h{subscr_id};
	my $ve_id = $h{ve_id};
	my $tool_id = $h{tool_id};
	my $zone = $h{zone};
	my $url_ext = $h{url_ext};

	## find person
	$person_id = HSPC::MT::Core->find_active_person_by_acc(
		account_id=>$account_no
	);

	return $ENV{system_obj}->exit_noobject() unless ($person_id);
	## find tool
	unless ( $tool_id ) {
		if ( $account_no ) {
			my $acc_type = HSPC::MT::Core->find_account_type($account_no);
			sw_die("Cannot find account type for #$account_no") unless (
				defined($acc_type)
			);
			if ( $acc_type == &SW_HSP ) {
				$tool_id = 'pcc';
			} elsif ( $acc_type == &SW_RESELLER ) {
				$tool_id = 'rcc';
			} else {
				$tool_id = 'cp';
			}
		} else {
			$tool_id = '';
		}
	}
	$self->pretender_on(person_id=>$person_id,tool_id=>$tool_id);
	
	$ENV{security_obj}->set_account($account_no) if ( $account_no );
	$ENV{system_obj}->upref_account(
		account=>$account_no,
		tool_id=>$tool_id
	);

	return sw_layout_redirect_local("/$tool_id") unless ( $tool_id eq 'cp');

	## for cp set preference
	return sw_layout_redirect_local(
		"/cp/index.cgi/top/zone,system/ve_id,$ve_id"
	) if ( $ve_id );

	return sw_layout_redirect_local(
		"/cp/index.cgi/top/zone,system/subscr_id,$subscr_id"
	) if ( $subscr_id );

	return sw_layout_redirect_local(
		"/cp/index.cgi/top/zone,account/acc_id,$account_no"
	) if ( $account_no );

	return sw_layout_redirect_local(
		"/cp/index.cgi/top/zone,$zone"
	) if ( $zone );

	return sw_layout_redirect_local(
		"/$tool_id/$url_ext"
	) if ( $url_ext );

	return sw_layout_redirect_local("/cp/");
}##/redirect_to_tool


## >> self
## =>>tab
## =>>id
sub get_tool_tip {
	my $self = shift;
	my %h = (tab=>undef,id=>undef,@_);
	my $tab = $h{tab} || sw_die("HSPC::WebSystem->get_tool_tip: tab is required");
	my $id  = $h{id} || sw_die("HSPC::WebSystem->get_tool_tip: id is required");

	my $show_tip = $self->get_user_option(option => 'is_menu_tool_tips_enabled');
	return undef unless ($show_tip);

	## XXX amark@ please complete 'tips' feature. It works realy slowly.
	## To avoid reading and parsing xml file for each link
	## local tips cache was added
	## Please investigate it to find better solution if bugs will appear.
	if(	!ref($ENV{security_obj}->{__local_tips_cache}) 
	||  !ref($ENV{security_obj}->{__local_tips_cache}->{$tab})
	) {	
		my $file = &sw_find_custom_file(
			base_path  =>&TIPS_PREFIX,
			file       =>"$tab.xml",
			provider_id=>$ENV{security_obj}->account_obj->vendor_id()
		);
		return undef unless ( -f $file );

		my $res = HSPC::LocResource->new($file,$LOCALE{local_lang});
		$ENV{security_obj}->{__local_tips_cache}->{$tab} = $res->strings();
	}
	return $ENV{security_obj}->{__local_tips_cache}->{$tab}->{$id};
	
}##/get_tool_tip

## >> self
## =>>option
sub get_user_option {
	my $self = shift;
	my %h = (@_);
	my $option = $h{'option'} || sw_die("HSPC::WebSystem->is_user_option_enabled: option is required");

	my $value = $ENV{system_obj}->upref_var($option);
	unless (defined $value) {
		my $account_obj = $ENV{security_obj}->account_obj or return undef;
		my $config = HSPC::MT::GM->find_provider_config(
			id	=> $account_obj->vendor_id()
		);
		$value = $config->{$option};
		$ENV{system_obj}->upref_var($option,$value);
	}
	
	return $value;
}##/get_user_option


## >> self
## =>>tab
sub get_loading_bar {
	my $self = shift;

	return $self->get_user_option(option => 'is_loading_bar_enabled');
}##/get_loading_bar

## >> self
## =>>url
sub download_file {
	my $self = shift;
	my %h = (
		url => undef,
		@_
	);

	$self->{download_file_url} = '' unless ( exists $self->{download_file_url} );

	if ($h{'url'}) {
		$self->{download_file_url} = $h{'url'}
	}
	
	return $self->{download_file_url} ? $self->{download_file_url} : undef;
}##/download_file

## >> self
## =>>url
sub open_popup {
	my $self = shift;
	my %h = (
		url => undef,
		width => 400,
		height => 450,
		@_
	);

	$self->{popup_url} = '' unless ( exists $self->{popup_url} );

	if ($h{'url'}) {
		$self->{popup_url} = $h{'url'};
		$self->{popup_width} = $h{'width'};
		$self->{popup_height} = $h{'height'};
	}

	my $result = undef;
	if ( $self->{popup_url} ) {
		$result = qq|window.open('$self->{popup_url}','','toolbar=0,location=0,resizable=1,scrollbars=1,width=$self->{popup_width},height=$self->{popup_height}');|;
	}
	
	return $result;
}##/open_popup

##############################################################################

# >>  class
# =>> look HSPC::System for other arguments
# =>  tool_id
sub new{	
	my ($class,%h) = @_;

	## Prevent from double initialization here
	my $self = $class->SUPER::new(%h);

	###### prepare internal vars
	$self->{_upref} = {};					
	$self->{_tool_id} = $h{tool_id};


	$self;
}




# < ref to hash with keys
#    product_id =>  '01'
#    product_caption =>  'Parallels Business Automation - Standard'
#    product_version =>  '2.0;
sub product_info{
	my $self = shift;
	return {
		product_id => SYSTEM_PRODUCT_ID,
		product_code => SYSTEM_PRODUCT_CAPTION,
		product_version => SYSTEM_PRODUCT_VERSION,
	};
}

## >> self
## << return current Provider ID
sub provider_id{
	my $self = shift;

	my $bid  = $ENV{HSPC_CU_SKIN_BID};
	return $bid if defined $bid;

	if($ENV{layout_obj} && $ENV{layout_obj}->{bid}){
		## bid existed in URL, rewrite bid
		my $hiding_bid = $self->_check_provider_server_name();
		if(!$hiding_bid || $hiding_bid == &SW_HSP_ID) { 
			$bid = $ENV{layout_obj}->{bid};
		}
		else {
			$bid = $hiding_bid;
		}
	}elsif( defined $ENV{security_obj} ){
		my $sec = $ENV{security_obj};
		if( $sec->account_type == &SW_CUSTOMER
			|| $self->tool_id eq 'cp'
		){
			## it is a customer or in CP, we need his vendor ID
			my $vendor_id = HSPC::MT::Core->find_account_vendor_id($sec->account_no);
			die 'ERROR : can`t load account for account_no '.$sec->account_no unless defined $vendor_id;
			$bid = $vendor_id;
		}else{
			## it is a Provider or Reseller
			$bid = $ENV{security_obj}->account_no if ($ENV{security_obj});
		}
	}			
	###### bid undefined
	unless( defined $bid ){ 
		$bid = $self->check_provider_server_name()||1;
	}
	###### store bid
	$ENV{HSPC_CU_SKIN_BID} = $bid;
	return $bid;		
}

## << logged person email 
sub person_email{
	my $self = shift;
	return undef unless $ENV{security_obj}; 	
	my $p = $ENV{security_obj}->person_obj or return undef;
	return $p->email;
}

## >> self
## << GMT offset
sub locale_lang{
	my $self = shift;

	my $check_session = 0;
	my $tool_id = $self->tool_id();
	$check_session = 1 if (
		!$tool_id || $tool_id eq 'store' || $tool_id eq 'site'
	);
	unless ( $self->{_lang} ) {
		if ( $check_session && $ENV{session}->{LANG} ) {
			$self->{_lang} = $ENV{session}->{LANG};
		} elsif ( !($ENV{security_obj} && $ENV{security_obj}->is_signed_in()) ) {
			$self->{_lang} = $ENV{session}->{login_lang} || $self->provider_config->def_lang();
		} else {
			if ( $ENV{security_obj}->person_obj() ) {
				$self->{_lang} = $ENV{security_obj}->person_obj->lang();
			} else {
				$self->{_lang} = $ENV{session}->{login_lang} || $self->provider_config->def_lang();
			}
		}
	}
	## TODO check if sw_argparam('_lang') is valid lang
	$self->{_lang} = sw_argparam('_lang') if (sw_argparam('_lang'));
	$ENV{session}->{LANG} = $self->{_lang} if ( $check_session && $ENV{session} );
	return $self->{_lang};
}

## >> self
## << timezone ID
sub locale_timezone{
	my $self = shift;	
	return $self->provider_config->def_timezone
		unless $ENV{security_obj} && $ENV{security_obj}->is_signed_in;
	return $ENV{security_obj}->person_obj->timezone;
}


### >> self
### << boolean : YES for hiding of <base href="..."  tag, NO(default) for showing
sub is_basehref_disabled{
	my $self = shift;
	return $self->{is_basehref_disabled};
}

### >> self
sub disable_basehref{
	my $self = shift;
	$self->{is_basehref_disabled} = 1;
	return
}

### >> self
sub enable_basehref{
	my $self = shift;
	$self->{is_basehref_disabled} = undef;
	return
}

#### init ( or reinit ) security object
## >> self
## => force_login : force login for nonlogged person
sub reinit_security{
	my $self = shift;
	my %h = (@_);
	
	### compartible with old code
	my $system_obj  = $self;
	
	# init security	
	my $security_obj = $ENV{security_obj} = HSPC::WebSecurity->factory (
		SESSION		=> $ENV{session},
		LANGUAGE	=> $LOCALE{local_lang},
		REQUIRE		=> ! $h{force_login}? 0 : 1,		
		RETURNURL	=> $h{force_login} ? (($ENV{HTTPS} eq 'on' ? 'https://' : 'http://').&sw_web_server_name.'/'.$system_obj->tool_baseurl):undef,
		DEBUG		=> 0
	);

	###### load user preferences from session or DB
	$ENV{HSPC_SYSTEM}->restore_upref;
	HSPC::Localization::set_language( $system_obj->locale_lang);

	# skip security checking for public sites #	
	return 1 unless $h{force_login};

	# redirect to login #
	unless ($security_obj->is_signed_in) {
		my $self_url;
		if (&sw_web_server_port eq $CONFIG{'BG_SERVER_NOSSL_PORT'}) {
			$self_url = 'http://'.&sw_web_server_name.$ENV{'REQUEST_URI'};
		} elsif  (&sw_web_server_port eq $CONFIG{'BG_SERVER_SSL_PORT'}) {
			$self_url = 'https://'.&sw_web_server_name.$ENV{'REQUEST_URI'};
		} else { 
			$self_url = $ENV{q}->self_url();
		}				
		my $node = $ENV{COMPREP} && $ENV{COMPREP}->node;
		my $url = $ENV{COMPREP}
			? $ENV{COMPREP}->link(alias => 'login', $node
				? {frame_url => sw_encode_base64_safe($ENV{COMPREP}->link(sw_uriparams()))}
				: ())
			: '/' . $self->tool_baseurl . '/login.cgi?ret_url='
				. sw_encode_base64_safe($self_url);
		return sw_web_redirect_local(url => $url);
	}

	{# set current account and check it #								
		my $account_type = $ENV{HSPC_SYSTEM}->tool_account_types;
		
		my $account;
		my $dbh = $self->db_connection();
																										
		# try to activate last used account #				
		if ( $self->tool_baseurl eq 'cp' || $self->tool_baseurl eq 'store' || $self->tool_baseurl eq 'mycp') {
			$account	= $system_obj->upref_account();
			$security_obj->set_account($account) if (defined $account);
		}
		my $res = $security_obj->check($dbh);
		HSPC::Logger->debug(qq|LOGIN:try to login to account # $account, result=$res|);
		if (defined($res) && !$account) {
			$account = $res->account_no();
		} elsif (!$res) {
			undef $account;
		}
		# oops....can`t set last used account , try to set another #
		unless($account){		
			$self->pretender_off() if ( $self->is_pretender_on() );
			my $all_accounts = $security_obj->get_accounts();				
			
			# convert account_type to array
			my $types = ref($account_type) eq 'ARRAY'?$account_type:[$account_type];
			
			# CYCLE FOR ALL AVAILABLE ACCOUNT TYPES #
			my $check_res;
			foreach my $type( @$types ){										
				my $accounts = $all_accounts->{ $type };		
				# try to activate any available account #		
				foreach ( @$accounts ){											
					$security_obj->set_account($_);		
					$check_res = $security_obj->check($dbh);
					HSPC::Logger->debug(qq|LOGIN: try to login to account # $_, result=$check_res|);
					if( $check_res ){
						$account = $_;
						last;					
					}
				}						
				last if defined $account;
			}
			# /CYCLE FOR ALL AVAILABLE ACCOUNT TYPES #
		}		
		
		# check result #
		if(not defined $account){
			my $all_accs = $security_obj->get_accounts();
			my $c;			
			foreach my $type(keys %$all_accs){
				$c.=$type.'(';
				$c.=join(';',@{$all_accs->{$type}});
				$c.=')';
			}

			my $pretender_key = $self->pretender_key($self->tool_id());
			HSPC::Logger->debug(qq|LOGIN: can not login into tool :|.$ENV{HSPC_SYSTEM}->tool_id.q|, all accounts=|.$c.q| person_id=|.$security_obj->person_id.q| pret=|.$ENV{session}->{$pretender_key});
			return sw_web_forbidden();
		}
		
		# store activated account #
		$ENV{HSPC_SYSTEM}->upref_account(account=>$account);
	}
	##
	$ENV{HSPC_CU_SKIN_BID} =  undef;
	$system_obj->provider_id();
}

## reinits session by session ID
sub reinit_session {
	my $self = shift;
	my %h = (@_);
	&sw_die('Session was not specified') unless $h{session_id};

        delete $ENV{session};

	my $cookie_in = $ENV{cookies_in} || {};
	my $cookie_out = $ENV{cookies_out} || {};
	$ENV{session} = HSPC::Session::checkSession($cookie_in,$cookie_out,undef,$h{session_id});
}

sub restore_upref{
	my $self = shift;

	$self->{_upref} = $self->restore_data(key=>'HSPC_UPREF');
		
	if( (not defined $self->{_upref}) 
		&& $ENV{security_obj} 
		&& $ENV{security_obj}->person_id>0
	){

		# restore upref from db #
		$self->{_upref} = undef;
		my $person_id;
		$person_id = $ENV{security_obj}->person_id if ($ENV{security_obj});
		my $vars = HSPC::WebDB::select_hashrows('SELECT * FROM core_upref_var WHERE person_id=?',$person_id);
		foreach my $var(@$vars){
			next unless ref($var) eq 'HASH';
			if(defined $var->{sub_id} && $var->{sub_id} ne ''){				
				unless(defined $self->{_upref}->{ $var->{id} }){
					$self->{_upref}->{ $var->{id} } = {};
				}				
				next unless ref($self->{_upref}->{ $var->{id} }) eq 'HASH';				
				$self->{_upref}->{ $var->{id} }->{ $var->{sub_id}  } = $var->{value};				
			}else{
				$self->{_upref}->{ $var->{id} } = $var->{value};
			}
		}
		
		# store upref 
		$self->store_data(key=>'HSPC_UPREF',data=>$self->{_upref});
	}
	
	# CHECK CURRENT LANGUAGE #
	$self->{_upref}->{lang} = $LOCALE{def_lang} if not defined $self->{_upref}->{lang};
}

## >> self
## << ref to HSPC::MT::CU::Skin instance
sub ui_skin{
	my $self = shift;	
	return $self->{_ui_skin};
}

## >> self
## << current skin version
sub ui_skin_version{
	my $self = shift;	
	return $self->{_ui_palette_version};
}

sub tool_account_types{
	my $self = shift;
	
	return undef unless $self->tool_id;
	return $self->{_tool_account_types};
}

# < tool_id
sub tool_id{
	my $self = shift;
	my $tool_id = $self->{_tool_id};
	return $tool_id if defined $tool_id && $tool_id ne '';

	my $def_url = $self->{_tool_baseurl} = $ENV{COMPREP} &&  $ENV{COMPREP}->{tool_id}
		|| $ENV{REQUEST_URI} =~ m#^/([^/]+)(/|$)# && lc($1);

	my $tool = HSPC::MT::CU->find_tool_by_def_url($def_url);
	return undef unless $tool;

	$self->{_tool_id} = $tool->id();
	$self->{_tool_title} = $tool->title();
	$self->{_tool_account_types} = $tool->account_types();
	$self->{_tool} = $tool;

	return $self->{_tool_id};
}

# < tool_id
# > pretender_key
sub pretender_key {
	my $self = shift;
	my $tool_id = shift;
	my $pretender_key = 'HSPC_PRETENDER';
	if ( $tool_id ) {
		$tool_id = 'cp' if ( $tool_id eq 'cp' || $tool_id eq 'store' || $tool_id eq 'mycp' );
		$pretender_key .= "_$tool_id";
	}
	return $pretender_key;
}

# < return to current HSPC::MT::CU::Tool
sub tool{
	my $self = shift;		
	return undef unless $self->tool_id;	
	return $self->{_tool};
}

sub tool_title{
	my $self = shift;		
	return undef unless $self->tool_id;	
	return $self->{_tool_title};
}

## < return tool base_url, for example : "pcc","site"
sub tool_baseurl{
	my $self = shift;		
	return undef unless $self->tool_id;	
	return $self->{_tool_baseurl};
}

## >> self
## >> new_value
## << current_value
sub upref_pc{
	my ($self,$value) = @_;			

	if( defined  $value){
		return $self->{_upref}->{pc} if $self->{_upref}->{pc} eq $value;
		$self->{_upref}->{pc} = $value;
		$self->_store_upref();		
	}
	return $self->{_upref}->{pc};
}


# >= account #REQ_N# set new account
# >= tool_id #REQ_N# get account for special type
# < account : account number for req.type
sub upref_account{
	my ($self,%h) = @_;			

	my $tool_id = $h{tool_id} || $self->tool_id;
	
	$self->{'_upref'}->{'accounts'} = {} unless ref($self->{'_upref'}->{'accounts'});
	
	if( defined $h{account}){	
		my $old_value = $self->{_upref}->{accounts}->{ $tool_id };
		return $old_value if $old_value eq $h{account};
		
		$self->{_upref}->{accounts}->{ $tool_id } = $h{account};		
		$self->_store_upref();		
		##
		HSPC::Logger->debug("SYSTEM: set account \#$h{account},tool \#$tool_id,user #".$ENV{security_obj}->person_id);
	}			
		
	return $self->{_upref}->{accounts}->{ $h{tool_id} } if defined $h{tool_id};
	return $self->{_upref}->{accounts}->{ $tool_id };	
}##/upref_account


# >> self
# >= name
# >= value
# >= alias
# >= no_save - don't store in DB
# <  value
sub upref_by_alias{
	my $self = shift;
	my %h = (
		name	=> undef,
		value	=> undef,
		alias	=> undef,
		no_save	=> undef,
		@_
	);

	die "SYSTEM: upref_by_alias => name not set" 
		unless defined $h{name};

	## Current screen alias
	$h{alias} ||= $ENV{COMPREP} && $ENV{COMPREP}->node_alias;

	return undef
		unless defined $h{alias};

	$self->{'_upref'}->{$h{name}} = {}
		unless ref($self->{'_upref'}->{$h{name}});
	
	if( defined $h{value}){	
		my $old_value = $self->{_upref}->{$h{name}}->{$h{alias}};
		return $old_value 
			if $old_value eq $h{value};

		$self->{_upref}->{$h{name}}->{$h{alias}} = $h{value};
		$self->_store_upref( alias => $h{alias} )
			unless $h{no_save};
	}			
		
	return $self->{_upref}->{$h{name}}->{ $h{alias} };	
}##/upref_by_alias

# >> self
# >= alias
sub store_upref_by_alias {
	my ($self,$alias) = @_;

	die "SYSTEM: store_upref_by_alias => alias not set" 
		unless defined $alias;

	$self->_store_upref( alias => $alias );
}

# >> self
# >> name
# >> value
# <  value
sub upref_var{
	my ($self,$name,$value) = @_;			
	die unless defined $name;

	if( defined  $value){
		return $self->{_upref}->{$name} if 
			$self->{_upref}->{$name} eq $value;
		$self->{_upref}->{$name} = $value;
		$self->_store_upref(var_id=>$name);		
	}
	return $self->{_upref}->{$name};
}

sub ui_flush_cache{	
	my $self = shift;
	
	$ui_cache_share->store( $ui_cache_share->fetch() +1 );
}

# < colors hash
sub ui_palette{
	my $self = shift;
	
	$self->_ui_init() unless $self->{_ui_palette};
	return $self->{_ui_palette};
}

# < images hash
sub ui_images{
	my $self = shift;
	
	$self->_ui_init() unless defined $self->{_ui_images};
		
	return $self->{_ui_images};
}


# =>> id  : ImageType ID
# => border
# < <img src=".." ..>
sub ui_image_src{
	my ($self)=shift;		
	my %h=(
		border=>0,
		align=>undef,
		vspace=>undef,
		hspace=>undef,
		@_
	);
	
	if(defined $h{id}){
		$h{id} = $h{id};
	}else{
		sw_die('ERROR: ui_image_src(), id=> undefined');		
	}	
	my $image = $self->ui_images()->{$h{id}};

	
	if(!$image){
		HSPC::Logger->warn('[WEBSYSTEM] ERROR: can`t find image source '.$h{id});
		return EMPTY_IMAGE_URL;
	}	

	## user defined attributes
	my %custom_attr = map {$_=>$h{$_}} grep {$_ ne 'id' && defined $h{$_}} keys %h;
	
	my $path = $self->ui_image_path(id => $h{id});
	## image attributes
	my %image_attr = (
		src => $path, 
		width => $image->{image_width},
		height => $image->{image_height}, 
		($image->{image_alt} ne '' ? (alt => $image->{image_alt}):()),
		%custom_attr);
	return q|<img |.join(' ',map {qq|$_="$image_attr{$_}"|} ('src',grep {$_ ne 'src'} keys %image_attr)).q|>|;
}

# =>> id  : ImageType ID
# < ref to hash {path name image_width image_height image_alt}
sub ui_image{
	my ($self)=shift;		
	my %h=(
		@_
	);		

	## If no site id - we assume its public site and 
	## will take image from public site repository
			
	if(defined $h{id}){
		$h{id} = $h{id};
	}else{
		sw_die('ERROR: ui_image_src(), undefined =>>id');		
	}	
	
	my $image = $self->ui_images()->{$h{id}};
	
	if(!$image){
		HSPC::Logger->warn('[WEBSYSTEM] ERROR: can`t find image '.$h{id});
		return {
				path=>'/images/',name=>'1.gif',image_width=>'1',
				image_height=>'1',image_alt=>'Image not found'
		};
	} else {
		my $path = $self->ui_image_path(id => $h{id});
		$path =~ s|$image->{name}||;
		$image->{path} = $path;
	}
	
	return $image;
}

# =>> id  : ImageType ID
# =>> skip_error : 1=return undef for unfound image,0/undef=return 1.gif
# < path  or undef
sub ui_image_path{
	my $self=shift;			
	my %h=(
		@_
	);
			
	if(defined $h{id}){		
		$h{id} = $h{id};		
	}else{
		sw_die('ERROR: ui_image_path(), undefined =>>id');		
	}		
	
	my $images = $self->ui_images;	
	my $image = $images->{$h{id}};	
	
	if(!$image){
		return undef if $h{skip_error};
		HSPC::Logger->warn('[WEBSYSTEM] can`t find image path '.$h{id});
		return '/images/1.gif';
	}
	
	
	my $path;
	my $skin;
	## Get skin object from arguments or from enviroment
	if ($h{skin} && $h{skin}->id()) {
		$skin = $h{skin};
	} else {
		$skin = $ENV{skin_obj};
		$skin = HSPC::MT::CU->find_default_skin() unless ($skin && $skin->id());
		$ENV{skin_obj} = $skin;
	}
	## Search image using skin object
	$path = $skin->find_resource_path( rel_path => $image->{path}.$image->{name});
	$path =~ s|$CONFIG{HSPCROOT_ROOT}||;## remove hspc-root folder from path
	return $path;
}

# >> self
# =>> SOURCE : template_path
# =>> any arguments for Text::Template
# < ref to Text::Template instance
sub find_text_template{
	my ($self,%args) = @_;
	my $src = $args{SOURCE};
	## string identifying state of the file based on 
	## file size in bytes and last modification time
	my $file_id = join(':',(stat($src))[7,9]);
	
	my $template = $CONFIG{UI_TEMPLATES_CACHING} &&
		$cache_ui_templates_stat{$src} eq $file_id ? $cache_ui_templates{$src} : undef;
	unless( defined $template ){
		$template = $self->SUPER::find_text_template(%args);

		if($CONFIG{UI_CACHING_DEBUG}){
			HSPC::Logger->debug("system: load template '$src'");
		}
		if ($CONFIG{UI_TEMPLATES_CACHING}) {
			$cache_ui_templates{$src} = $template;
			$cache_ui_templates_stat{$src} = $file_id;
		}
	}
	return $template;
}

## >>  self
## =>> person_id
## =>> tool_id
## << undef on error, person_id on success
sub pretender_on{
	my ($self, %h, $error_code) = @_;
	my $tool_id = $h{tool_id} || $self->tool_id();
	my $pretender_key = $self->pretender_key($tool_id);
	sw_die('undefined person_id') unless defined $h{person_id};
	sw_die('uninitialized $ENV{security_obj}') unless $ENV{security_obj};

	## $error_code holds error in range (1, 2) and checked at this sub return
	## 1 == permission violation
	## 2 == account absent

	if ( $h{person_id} == $ENV{security_obj}->person_id() ) {
		## pretemd on himself
		return ($ENV{session}->{$pretender_key} = $h{person_id});
	}
	
	local $ENV{COMPREP} = HSPC::CompRep->new(tool => 'pcc');

	my $acc_list = $ENV{security_obj}->get_accounts( person_id => $ENV{security_obj}->auth_person_id );
		
	if (ref($acc_list->{&SW_HSP}) && @{$acc_list->{&SW_HSP}} ) {
		## has appropriate role for provider
		return ($ENV{session}->{$pretender_key} = $h{person_id});
	}

	my $person = HSPC::MT::Core->find_person(id=>$h{person_id});
	my $p_acc_list = $person->accounts();

	if (ref($acc_list->{&SW_RESELLER}) && @{$acc_list->{&SW_RESELLER}}) {
		$error_code++;
		## has appropriate role in reseller
		if ( grep { $_->{vendor_id} == $acc_list->{&SW_RESELLER}->[0] && $_->{status} == 0 } @{$p_acc_list} ) {
			return ($ENV{session}->{$pretender_key} = $h{person_id});
		}
	}
	
	$error_code--;

	my @errors = ('cant_switch_2acc_permissions', 'cant_switch_2acc_deleted');
	HSPC::Logger->info( string($errors[$error_code]) );
	return;
}

## >>  self
## => tool_id
## << 1/0
sub is_pretender_on{
	my ($self,%h) = @_;
	my $pretender_key = $self->pretender_key($h{tool_id});
	return defined $ENV{session}->{$pretender_key};
}


## >>  self
## => tool_id
sub pretender_off{
	my ($self,%h) = @_;
	my $pretender_key = $self->pretender_key($h{tool_id});
	return 0 unless defined $ENV{session}->{$pretender_key};
		
	################## pretender_off
	$ENV{session}->{$pretender_key} = undef;
	## restore person
	$ENV{security_obj}->person_id($ENV{security_obj}->auth_person_id());
	
	return 1;
}


## => self
## <= screen ID
sub generate_screen_id {
	my $self = shift;
	my %h = @_;				

	my $node = $ENV{COMPREP} && $ENV{COMPREP}->node;
	return join('.', $self->tool->num_id, (split(/\./, $node->{uri}), '00', '00')[0..2])
		if $node;

	## backward compatibility
	if (defined $ENV{layout}) {
		my $unit_id = $ENV{layout}->unit_id;
		return $self->tool->num_id . '.' . ($unit_id ? $unit_id . '.' : '') . $h{devscreen_id};		
	} else {		
		return $self->tool->num_id . '.' . $h{devscreen_id};		
	}	
}

# >>  self
# =>> screen_id : tool_id/unit_id/app_id/page_id/field_id, look HelpManager doc 
#    or
# =>> devscreen_id : DevScreenID
#				for layout apps : app_id.page_id#field_id, look HelpManager doc 
#				for cgi-scipts : unit_id.app_id.page_id#field_id
# < HelpScreen_ID
sub generate_helpscreen_id{
	my ($self,%h) = @_;		
	
	unless(defined $h{screen_id}){		
		die unless defined $h{devscreen_id};
		##$h{devscreen_id} = '00' unless defined $h{devscreen_id};
		$h{screen_id} = $self->generate_screen_id(devscreen_id=>$h{devscreen_id});
	}
	return SYSTEM_PRODUCT_ID.'.'.$h{screen_id};	
	#return SYSTEM_PRODUCT_CODE.'.'.SYSTEM_PRODUCT_VERSION.'.'.$self->locale_lang.'.'.$h{screen_id};	
	#return $h{screen_id};	
}


## >> self
## =>>owner_id
## < background
sub site_background {
	my $self = shift;
	my %h = (owner_id=>undef,@_);
	my $owner_id = $h{owner_id} || sw_die(
		"HSPC::WebSystem->site_background: owner_id is requred"
	);
	my $site = HSPC::MT::PM->find_site(
		owner_id=>$ENV{system_obj}->provider_config()->id(),need=>1
	);

	my $res;
	if( $site->img_bkg_id() ) {
		my $bg_file = HSPC::MT::CU->find_file(id=>$site->img_bkg_id());
		$res = $bg_file->path().$bg_file->name() if ( $bg_file );
	} else {
		$res = "<imgpath_title_bg_site>";
	}
	return $res;
}##/site_background



## >> self
## =>>owner_id
## < logo
sub site_logo {
	my $self = shift;
	my %h = (owner_id=>undef,@_);
	my $owner_id = $h{owner_id} || sw_die(
		"HSPC::WebSystem->site_logo: owner_id is requred"
	);
	my $site = HSPC::MT::PM->find_site(
		owner_id=>$ENV{system_obj}->provider_config()->id(),need=>1
	);

	my $res;
	if (($site->is_textlogo()) && ($site->textlogo)) {
		$res .= $site->textlogo->content;
	} else {
		if ( $site->img_logo_id() ) {
			my $logo_file = HSPC::MT::CU->find_file(id=>$site->img_logo_id());
			$res = $logo_file->img_html_tag() if ( $logo_file );
		} else {
			$res = $self->ui_image_src(id=>'logo_store');
		}
	}
	return $res;
}##/site_logo

###### return common report for system_die() method, can be redefined in child class
## >> self
## << report
sub _generate_bugreport_common{
	my $self = shift;
	my $report = $self->SUPER::_generate_bugreport_common(@_);

	##$report .= "ENV{PATH_INFO}: ".$ENV{PATH_INFO}."\n";
	$report .= "ENV{SERVER_NAME}: ".&sw_web_server_name."\n";
	$report .= "ENV{REQUEST_URI}: ".$ENV{REQUEST_URI}."\n";
	$report .= "URL: " . ($ENV{q}->can('self_url')
			? $ENV{q}->self_url : $ENV{REQUEST}) . "\n"
		if $ENV{q};
	$report .= "SESSION_ID: "
		.($ENV{session}?$ENV{session}->{_session_id}:'undefined')
		."\n";
	return $report;
}

###### return full report for system_die() method, can be redefined in child class
## >> self
## << report
sub _generate_bugreport_full{
	my $self = shift;
	my $report = $self->SUPER::_generate_bugreport_full(@_);
	
	########### report security details
	$report .= "------- SECURITY REPORT--------\n";
	my $sec = $ENV{security_obj};
	if( $sec ){
		$report .= "account_id=".$sec->account_no."\n";
		$report .= "account_type=".$sec->account_type."\n";
	}else{
		$report .= "security object undefined\n";
	}
	$report .= "------- /SECURITY REPORT--------\n";
	
	########### report session details
	##$report .= "------- SESSION REPORT--------\n";
	##$report .= Dumper( $ENV{session} );
	##$report .= "------- /SESSION REPORT--------\n";
	
	return $report;
}

sub _ui_check_skins{
	my $self = shift;
	
	return undef if $cache_ui_skin_check == $ui_cache_share->fetch;
	%cache_ui_palettes=();
	%cache_ui_palettes_as=();	
	
	$cache_ui_skin_check =  $ui_cache_share->fetch;
}


## Function returns HSPC::MT::CU::Skin object
## >> self
## << HSPC::MT::CU::Skin
sub _ui_find_skin{
	my $self = shift;
	
	return $ENV{skin_obj} if ($ENV{skin_obj});
	return HSPC::MT::CU->find_default_skin();
}

sub _ui_init{
	my $self = shift;
	my $skin = $ENV{skin_obj};
	
	# skin was not found, ok, load default skin #
	$skin = HSPC::MT::CU->find_default_skin() unless $skin;
	die unless $skin;
	
	$self->{_ui_skin} = $skin;
			
	# load palette #				
	my $palette = $skin->palette;
	die unless $palette;
	$self->{_ui_palette} = $palette->colors_hash;		
	$self->{_ui_palette_version} = $palette->version;			
	
	# load images #	
	$self->{_ui_images} = $skin->images_info(cache=>$CONFIG{UI_SKIN_CACHING});
			
	if($CONFIG{UI_IMAGES_DEBUG}){		
		foreach( keys %{$self->{_ui_images}} ){
			my $image = $self->{_ui_images}->{$_};
			$image->{path}='/images/';
			$image->{name}='d1.gif';
		}
	}
		
}


## => var_id
## => var_subid
## => alias, to store data only for given alias
sub _store_upref{
	my ($self,%h) = @_;
	
	## store in session ##
	$self->store_data(key=>'HSPC_UPREF',data=>$self->{_upref});		
		
	if($ENV{security_obj} 
	&& $ENV{security_obj}->person_id > 0
	&& !$self->is_pretender_on(tool_id => $ENV{HSPC_SYSTEM}->tool_id)
	){## store in DB ##
		##
		my @vars;
		my $value;
		##
		if(defined $h{var_id}){
			push @vars,{
				id=>$h{var_id},
				subid=>$h{var_subid},
				value=>$self->{_upref}->{ $h{var_id} },
			};
		}else{
			foreach my $var_id( keys %{$self->{_upref}}  ){
				$value = $self->{_upref}->{$var_id};
				if(ref($value) eq 'HASH'){
					## store hash ##
					foreach(  keys %$value ){
						next if defined $h{alias} 
								&& $h{alias} ne $_;
						push @vars,{
							id=>$var_id,
							sub_id=>$_,
							value=>$value->{$_},
						}; 
					}
				}else{
					## store scalar ##
					push @vars,{
							id=>$var_id,
							sub_id=>'',
							value=>$self->{_upref}->{$var_id},						
					}; 
				}
			}
		}
		##
		my $person_id;
		$person_id = $ENV{security_obj}->person_id if ($ENV{security_obj});
		foreach my $var( @vars ){
			HSPC::WebDB::select_run(
				'REPLACE INTO core_upref_var (id,sub_id,person_id,value) VALUES (?,?,?,?)'
				,$var->{id},$var->{sub_id},$person_id,$var->{value}
			);
		}
		##/foreach
	}
}

# unescape URL-encoded data
sub _unescape {    
    my $todecode = shift;
    return undef unless defined($todecode);
    $todecode =~ tr/+/ /;       # pluses become spaces
    $todecode =~ s/%([0-9a-fA-F]{2})/pack("c",hex($1))/ge;
    return $todecode;
}

# URL-encode data
sub _escape {    
    my $toencode = shift;
    return undef unless defined($toencode);
    $toencode=~s/([^a-zA-Z0-9_.-])/uc sprintf("%%%02x",ord($1))/eg;
    return $toencode;
}

sub DESTROY{	
	my $self = shift;				
	
	##HSPC::Logger->debug( '[WEBSYSTEM] done request for url '.$ENV{q}->self_url() );
				
	if(defined $ENV{session}){
		untie %{$ENV{session}};	
		undef $ENV{session};
	}
	$self->SUPER::DESTROY(@_);
}


BEGIN{

	{
		# used during Apache restart only for setting rights
		# on SSL certificate/key files
		
		my (undef,undef,$root_id,undef) = getpwnam('root');
		my (undef,undef,undef,$apache_id) = getpwnam('apache');
		my $dir =
			(-e '/etc/apache2' ? '/etc/apache2' : '/etc/httpd') .
			'/conf/';
		my @dirs = qw(ssl.crt ssl.csr ssl.key);
		my @files = qw(ssl.crt/server.crt ssl.key/server.key);
		
		@dirs = map { $dir . $_ } @dirs;
		@files = map { $dir . $_ } @files;
			
		chown $root_id, $apache_id, @dirs, @files;
		chmod 0770, @dirs;
		chmod 0660, @files;		
	}

}

1;

__END__

=== PRIVATE SESSION VARIABLES ===

HSPC_UPREF

=== /SESSION VARIABLES ===

=== ENV VARIABLES ===

HSPC_CU_SKIN_BID  : current vendor number

=== /ENV_VARIABLES ===

==== HSPC::System object methods ====

public sub upref_pc(new_value) : set user_pref var "Items on page" or return current value
public sub upref_account(account=>) : set new current account or/and return current account
public sub locale_lang() : return current language
public sub tool_id() : return current tool_id
public sub ui_palette() : return palette hash
public sub ui_images() : return images hash

==== /HSPC::System object methods ====

==== HSPC::WebSystem object private attributes ====
	_upref : ref to hash with User Preferencies, keys :		
		accounts : user pref : ref to hash {site_acc_code=>account_no,...}
		pc : user pref : items to page count	
		lang : user pref : current language	
		.... : any custom keys
	_upref_changed : save upref
	_tool_id : id for HSPC::MT::CU::Tool object	
	_tool_title : HSPC::MT::CU::Tool title
	_tool_account_types : array allows account_types
	_tool_baseurl : current base URL ( 'cc'  or 'ec' or 'cp' or .... )
	_tool : ref to HSPC::MT::CU::Tool instance
	_ui_skin : ref to HSPC::MT::CU::Skin instance	
	_ui_palette  : 
	_ui_images : 
	__textout_modes : 
	__textout_offset : 
	__textout_offsetc : 
==== /HSPC::WebSystem object private attributes ====
